-- Require necessary modules
local interface = require("interface")
require("prototypes.init")

-- Initialize global variables
local function initialize_globals()
    global.heat_engines = global.heat_engines or {}
end

-- Register on_init and on_configuration_changed events
script.on_init(initialize_globals)
script.on_configuration_changed(initialize_globals)

-- Function to dynamically update input and output flow limits based on control data
local function update_flow_limits(energy_interface, energyOutput)
    -- Set input and output flow limits based on energy output
    energy_interface.electric_energy_source_prototype.input_flow_limit = energyOutput .. "W"
    energy_interface.electric_energy_source_prototype.output_flow_limit = energyOutput .. "W"
end

-- Function to update energy production based on control data
local function update_energy_production(energy_interface, energyOutput)
    -- Set energy production based on energy output
    energy_interface.electric_energy_source_prototype.production = energyOutput .. "W"
end

script.on_event(defines.events.on_built_entity, function(event)
    if event.created_entity.name == "heat-engine" then
        -- Create hidden energy interface
        local position = event.created_entity.position
        local surface = event.created_entity.surface
        local energy_interface = surface.create_entity({
            name = "hidden-energy-interface",
            type = "electric-energy-interface",
            position = {position.x + 1, position.y},
            force = event.created_entity.force,
            create_build_effect_smoke = false,
            render_player_index = event.player_index,
            visible = false
        })

        -- Check creation success
        if not energy_interface then
            game.print("Failed to create hidden energy interface.")
            return
        end

        -- Connect the energy interface to the heat engine
        energy_interface.connect_neighbour({
            wire = defines.wire_type.red,
            target_entity = event.created_entity
        })
        energy_interface.connect_neighbour({
            wire = defines.wire_type.green,
            target_entity = event.created_entity
        })

        -- Add to global heat_engines list
        table.insert(global.heat_engines, {engine = event.created_entity, interface = energy_interface})
    end
end)

script.on_nth_tick(60, function()
    for _, engine_data in pairs(global.heat_engines) do
        local engine = engine_data.engine
        local energy_interface = engine_data.interface
        if engine and engine.valid and energy_interface and energy_interface.valid then
            local temperature = engine.temperature -- Get temperature from the heat engine entity
            local energyOutput = interface.calculate_energy_output(temperature)

            update_flow_limits(energy_interface, energyOutput)
            update_energy_production(energy_interface, energyOutput)
        end
    end
end)