local pipe_covers = data.raw["pipe-cover"]

local function createConnections(connections, connection_type)
    local heat_connections = {}

    table.insert(heat_connections, {
        type = connection_type,
        position = {0, -1},
        direction = defines.direction.north
    })

    table.insert(heat_connections, {
        type = connection_type,
        position = {0, 1},
        direction = defines.direction.south
    })

    table.insert(heat_connections, {
        type = connection_type,
        position = {-1, 0},
        direction = defines.direction.west
    })

    table.insert(heat_connections, {
        type = connection_type,
        position = {1, 0},
        direction = defines.direction.east
    })

    return heat_connections
end

extend({
    {
        type = "electric-energy-interface",
        name = "hidden-energy-interface",
        icon = "__base__/graphics/icons/heat-exchanger.png",  -- Adjust the path to your icon
        icon_size = 64, icon_mipmaps = 4,
        flags = {"placeable-neutral", "player-creation"},
        minable = {mining_time = 0.1, result = "hidden-energy-interface"},
        max_health = 200,
        corpse = "small-remnants",
        collision_box = {{-0.9, -0.9}, {0.9, 0.9}},
        selection_box = {{-1, -1}, {1, 1}},
        energy_source = {
            type = "electric",
            buffer_capacity = "5MJ",
            usage_priority = "secondary-output",
            input_flow_limit = "0W",
            output_flow_limit = "0W"
        },
        energy_production = "0W",
        energy_usage = "0W",
        picture = {
            filename = "__base__/graphics/entity/heat-exchanger/heatex.png",
            priority = "extra-high",
            width = 64,
            height = 64,
            shift = {0.5, 0}
        }
    },

    {
        type = "heat-pipe",
        name = "heat-engine",
        icon = "__heat-engine__/graphics/icons/heat-engine.png",
        icon_size = 64,
        flags = {"placeable-neutral", "player-creation"},
        minable = {mining_time = 1, result = "heat-engine"},
        max_health = 300,
        corpse = "medium-remnants",
        dying_explosion = "medium-explosion",
        collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
        selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
        energy_source = {
            type = "electric",
            usage_priority = "primary-output"
        },
      heat_buffer = {
            max_temperature = 5000,
            specific_heat = "1GJ",
            heat_consumption_per_tick = "1KJ",
            max_transfer = "1GW",
            buffer_capacity = "1GJ",
            min_temperature = 15,
            connections = createConnections(connections, "heat")
        },
picture = {
    layers = {
        {
            filename = "__heat-engine__/graphics/entity/heat-engineH.png",
                line_length = 4,
                width = 120,
                height = 84,
				frame_count = 24,
				animation_speed = 0.75,
                shift = {0, 0},
		hr_version = {
                filename = "__heat-engine__/graphics/entity/heat-engineHR.png",
				line_length = 4,
                width = 120,
                height = 84,
				frame_count = 24,
				animation_speed = 0.75,
                shift = {0, 0},
                scale = 0.5
            }
        },
        {
            filename = "__base__/graphics/entity/heat-pipe-cover-north.png",
            width = 120,
            height = 84,
            shift = {0, -1},
            hr_version = {
                filename = "__base__/graphics/entity/hr-heat-pipe-cover-north.png",
                width = 240,
                height = 168,
                shift = {0, -1},
                scale = 0.5
            },
            tint = {r = 1, g = 1, b = 1, a = 0.5}
        },
        {
            filename = "__base__/graphics/entity/heat-pipe-cover-south.png",
            width = 120,
            height = 84,
            shift = {0, 1},
            hr_version = {
                filename = "__base__/graphics/entity/hr-heat-pipe-cover-south.png",
                width = 240,
                height = 168,
                shift = {0, 1},
                scale = 0.5
            },
            tint = {r = 1, g = 1, b = 1, a = 0.5}
        },
        {
            filename = "__base__/graphics/entity/heat-pipe-cover-west.png",
            width = 120,
            height = 84,
            shift = {-1, 0},
            hr_version = {
                filename = "__base__/graphics/entity/hr-heat-pipe-cover-west.png",
                width = 240,
                height = 168,
                shift = {-1, 0},
                scale = 0.5
            },
            tint = {r = 1, g = 1, b = 1, a = 0.5}
        },
        {
            filename = "__base__/graphics/entity/heat-pipe-cover-east.png",
            width = 120,
            height = 84,
            shift = {1, 0},
            hr_version = {
                filename = "__base__/graphics/entity/hr-heat-pipe-cover-east.png",
                width = 240,
                height = 168,
                shift = {1, 0},
                scale = 0.5
            },
            tint = {r = 1, g = 1, b = 1, a = 0.5}
        }
    }
},

        heat_glow_sprites ={
picture = {
    layers = {
        {
            filename = "__heat-engine__/graphics/entity/heat-engineHglow.png",
                line_length = 4,
                width = 120,
                height = 84,
				frame_count = 24,
				animation_speed = 0.75,
                shift = {0, 0},
            hr_version = {
                filename = "__heat-engine__/graphics/entity/heat-engineHRglow.png",
				line_length = 4,
                width = 120,
                height = 84,
				frame_count = 24,
				animation_speed = 0.75,
                shift = {0, 0},
                scale = 0.5
            }
        }
},
        heat_glow_light = {
            minimum_darkness = 0.3,
            color = { r = 1, g = 155/255, b = 0.05, a = 0 },
            shift = {-0.6, 3.5},
            size = 2.5,
            intensity = 0.3,
            add_perspective = true,
        }
    },
    {
        type = "int-setting",
        name = "heat-engine-conversion-factor",
        setting_type = "runtime-global",
        default_value = 200,
        minimum_value = 16,
        maximum_value = 5000,
        order = "a"
    },
    {
        type = "int-setting",
        name = "heat-engine-min-temperature",
        setting_type = "runtime-global",
        default_value = 15,
        minimum_value = 0,
        maximum_value = 100,
        order = "b"
    },
    {
        type = "item",
        name = "heat-engine",
        icon = "__heat-engine__/graphics/icons/heat-engine.png",
        subgroup = "energy",
        order = "b[steam-power]-d[heat-engine]",
        place_result = "heat-engine",
        icon_size = 64,
        stack_size = 10
    },
    {
        type = "recipe",
        name = "heat-engine",
        category = "crafting",  -- Adjust this category to match your desired crafting category
        normal = {
            ingredients = {
                {"steam-engine", 1},
                {"pipe", 5},
                {"iron-plate", 5}
            },
            result = "heat-engine"
        },
        expensive = {
            ingredients = {
                {"steam-engine", 2},
                {"pipe", 10},
                {"iron-plate", 10}
            },
            result = "heat-engine"
        }
    },
    {
        type = "technology",
        name = "heat-engine",
        icon = "__heat-engine__/graphics/icons/heat-engine.png",
        icon_size = 64,
        category = "basic-crafting",
        prerequisites = {"advanced-material-processing", "engine"},
        unit = {
            count = 100,
            ingredients = {
                {"automation-science-pack", 1},
                {"logistic-science-pack", 1}
            },
            time = 30
        },
        effects = {
            {
                type = "unlock-recipe",
                recipe = "heat-engine"
            }
        }
    }
  }
  }
})
